<?php

/**
 * E-goi.com
 *
 * @title      E-Goi Multi-channel Marketing
 * @package    E-Goi
 * @copyright  Copyright (c) 2012-2025 E-Goi - https://www.e-goi.com
 */

namespace Egoi\Marketing\Controller\Adminhtml\Account;

use Magento\Framework\App\Filesystem\DirectoryList;

/**
 * Class Bulk
 *
 * @package Egoi\Marketing\Controller\Adminhtml\Account
 */
class Bulk extends \Egoi\Marketing\Controller\Adminhtml\Account
{

    /**
     * @return \Magento\Framework\App\ResponseInterface|void
     * @throws \Exception
     */
    public function execute()
    {

        $resource = $this->_accountFactory->create()->getResource();
        $connection = $resource->getConnection();

        if ($this->getRequest()->getParam('export')) {
            $connection->update($resource->getTable('sales_order'), ['egoi_synced' => 0]);
            $this->messageManager->addSuccessMessage(__('Orders Data will be synced next time the cron runs'));

            return $this->_redirect('*/*/');
        }

        if ($this->getRequest()->getParam('export')) {
            try {
                $fileName = 'egoi-'.date('YmdHis').'.csv';
                $systemDir = $this->_fileSystem->getDirectoryWrite(DirectoryList::TMP);
                $file = $systemDir->getAbsolutePath($fileName);

                $fp = fopen($file, 'w');

                if ($fp === false) {
                    throw new \Magento\Framework\Exception\LocalizedException(__("Can't open dir for writing: %1 ",
                        $systemDir->getAbsolutePath()));
                }

                $subscribers = $this->egoi->addSubscriberBulk();

                $new = [];
                $keys = [];
                $sub = [];
                foreach ($subscribers as $key => $subscriber) {
                    foreach ($subscriber['base'] as $i => $info) {
                        $keys[$i] = null;
                        $new[$subscriber['base']['email']][$i] = $info;
                    };

                    if ( ! empty($subscriber['extra'])) {
                        foreach ($subscriber['extra'] as $i => $info) {
                            $code = $connection->fetchOne("SELECT attribute_code FROM "
                                .$resource->getTable('egoi_extra')
                                ." WHERE extra_code=? ", 'extra_'.$info['field_id']);
                            $new[$subscriber['base']['email']]['extra_'.$info['field_id']] = $info['value'];
                            $keys[$code] = null;
                        }
                    }
                }

                foreach ($new as $k => $subscriber) {
                    $sub[$k] = array_merge($keys, $subscriber);
                }

                fputcsv($fp, array_keys($keys));
                foreach ($sub as $fields) {
                    fputcsv($fp, $fields);
                }
                fclose($fp);
            } catch (\Exception $e) {
                $this->messageManager->addErrorMessage($e->getMessage());

                return $this->_redirect('*/*/');
            }

            return $this->_fileFactory->create($fileName, \file_get_contents($file), DirectoryList::TMP);
        }

        $resource->getConnection()
            ->update($resource->getTable('egoi_subscribers'), ['needs_sync' => 1]);

        $cron = $this->_cron->create();
        $data['status'] = 'pending';
        $data['job_code'] = 'egoi_mass_sync_customers';
        $data['scheduled_at'] = new \Zend_Db_Expr('NOW()');
        $data['created_at'] = new \Zend_Db_Expr('NOW()');
        $cron->setData($data)->save();
        $this->messageManager->addSuccessMessage(__('Data will be synced next time cron runs'));

        return $this->_redirect('*/*/');
    }
}
