<?php

/**
 * E-goi.com
 *
 * @title      E-Goi Multi-channel Marketing
 * @package    E-Goi
 * @copyright  Copyright (c) 2012-2025 E-Goi - https://www.e-goi.com
 */

namespace Egoi\Marketing\Model;

use Magento\Sales\Model\Order;
use Magento\Framework\App\Filesystem\DirectoryList;

/**
 * Class Egoi
 *
 * @package Egoi\Marketing\Model
 */
class Egoi extends \Magento\Framework\DataObject
{

    const EGOI_STATUS_SUBSCRIBED = 'active';

    const EGOI_STATUS_UNSUBSCRIBED = 'inactive';

    const STATUS_UNCONFIRMED = 'unconfirmed';

    const DEFAULT_QTY_TO_SYNC_CATALOG = 1000;

    /**
     *
     */
    const PLUGIN_KEY = 'e419a126e087bed65ad7fe8342f2f493';

    /**
     *
     */
    const ECOMMERCE_URL = 'https://api.egoiapp.com/';

    /**
     *
     */
    const API_URL_SOAP = 'http://api.e-goi.com/v2/soap.php?wsdl';

    /**
     *
     */
    const MOBILE_NUMBER_VALIDATION_URL = 'https://slingshot.egoiapp.com/api/v2/utilities/validate-phone';

    /**
     *
     */
    const MOBILE_NUMBER_SEND_URL = 'https://slingshot.egoiapp.com/api/v2/sms/messages/action/send';

    protected $orderStateMapping
        = [
            Order::STATE_NEW             => 'created',
            Order::STATE_PENDING_PAYMENT => 'created',
            Order::STATE_PROCESSING      => 'pending',
            Order::STATE_COMPLETE        => 'completed',
            Order::STATE_CLOSED          => 'completed',
            Order::STATE_CANCELED        => 'canceled',
            Order::STATE_HOLDED          => 'pending',
            Order::STATE_PAYMENT_REVIEW  => 'pending',
        ];

    /**
     * \Magento\Store\Model\App\Emulation
     */
    protected $emulation;

    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $storeManager;

    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * @var ListsFactory
     */
    protected $listsFactory;

    /**
     * @var AccountFactory
     */
    protected $accountFactory;

    /**
     * @var \Magento\Framework\Registry
     */
    protected $coreRegistry;

    /**
     * @var \Magento\Backend\Model\Auth\Session
     */
    protected $authSession;

    /**
     * @var SubscriberFactory
     */
    protected $subscriberFactory;

    /**
     * @var \Magento\Newsletter\Model\SubscriberFactory
     */
    protected $coreSubscriberFactory;

    /**
     * @var \Magento\Newsletter\Model\ResourceModel\Subscriber\CollectionFactory
     */
    protected $coreCollectionFactory;

    /**
     * @var ListsFactory
     */
    protected $extraFactory;

    /**
     * @var \Magento\Customer\Model\CustomerFactory
     */
    protected $customerFactory;

    /**
     * @var \Magento\Cron\Model\ScheduleFactory
     */
    protected $scheduleFactory;

    /**
     * @var \Magento\Framework\Filesystem
     */
    protected $filesystem;

    /**
     * @var \Magento\Framework\Url
     */
    protected $url;

    /**
     * @var \Magento\Sales\Model\OrderFactory
     */
    protected $orderFactory;

    /**
     * @var \Egoi\Marketing\Helper\Data
     */
    protected $egoiHelper;

    /**
     * @var \Magento\Catalog\Model\ProductFactory
     */
    protected $productFactory;

    /**
     * @var \Magento\Catalog\Model\CategoryFactory
     */
    protected $categoryFactory;

    /**
     * @var \Magento\Catalog\Helper\Image
     */
    protected $imageHelper;

    /**
     * @var \Magento\Framework\App\ReinitableConfig
     */
    protected $reinitableConfig;

    /**
     * @var \Magento\Config\Model\ResourceModel\Config
     */
    protected $config;

    /**
     * @var \Magento\ConfigurableProduct\Model\Product\Type\Configurable
     */
    protected $configurableType;

    /**
     * @var \Magento\Framework\Stdlib\DateTime\TimezoneInterface
     */
    protected $timezone;

    /**
     * @var \Magento\Checkout\Model\Session
     */
    protected $checkoutSession;

    /**
     * @param  \Magento\ConfigurableProduct\Model\Product\Type\Configurable  $configurableType
     * @param  \Magento\Framework\App\ReinitableConfig  $reinitableConfig
     * @param  \Magento\Config\Model\ResourceModel\Config  $config
     * @param  \Magento\Catalog\Helper\Image  $imageHelper
     * @param  \Magento\Catalog\Model\ProductFactory  $productFactory
     * @param  \Magento\Catalog\Model\CategoryFactory  $categoryFactory
     * @param  \Egoi\Marketing\Helper\Data  $egoiHelper
     * @param  \Magento\Sales\Model\OrderFactory  $orderFactory
     * @param  \Magento\Framework\Url  $urlHelper
     * @param  \Magento\Cron\Model\ScheduleFactory  $scheduleFactory
     * @param  \Magento\Newsletter\Model\SubscriberFactory  $coreSubscriberFactory
     * @param  \Magento\Newsletter\Model\ResourceModel\Subscriber\CollectionFactory  $coreCollectionFactory
     * @param  \Magento\Framework\Registry  $coreRegistry
     * @param  \Magento\Backend\Model\Auth\Session  $authSession
     * @param  \Magento\Framework\App\Config\ScopeConfigInterface  $scopeConfig
     * @param  \Magento\Store\Model\StoreManagerInterface  $storeManager
     * @param  \Magento\Customer\Model\CustomerFactory  $customerFactory
     * @param  ListsFactory  $listsFactory
     * @param  ExtraFactory  $extraFactory
     * @param  AccountFactory  $accountFactory
     * @param  \Magento\Framework\Filesystem  $filesystem
     * @param  SubscriberFactory  $subscribersFactory
     * @param  \Magento\Framework\Stdlib\DateTime\TimezoneInterface  $timezone
     * @param  \Magento\Checkout\Model\Session  $checkoutSession
     */
    public function __construct(
        \Magento\Store\Model\App\Emulation $emulation,
        \Magento\ConfigurableProduct\Model\Product\Type\Configurable $configurableType,
        \Magento\Framework\App\ReinitableConfig $reinitableConfig,
        \Magento\Config\Model\ResourceModel\Config $config,
        \Magento\Catalog\Helper\Image $imageHelper,
        \Magento\Catalog\Model\ProductFactory $productFactory,
        \Magento\Catalog\Model\CategoryFactory $categoryFactory,
        \Egoi\Marketing\Helper\Data $egoiHelper,
        \Magento\Sales\Model\OrderFactory $orderFactory,
        \Magento\Framework\Url $urlHelper,
        \Magento\Cron\Model\ScheduleFactory $scheduleFactory,
        \Magento\Newsletter\Model\SubscriberFactory $coreSubscriberFactory,
        \Magento\Newsletter\Model\ResourceModel\Subscriber\CollectionFactory $coreCollectionFactory,
        \Magento\Framework\Registry $coreRegistry,
        \Magento\Backend\Model\Auth\Session $authSession,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Customer\Model\CustomerFactory $customerFactory,
        ListsFactory $listsFactory,
        ExtraFactory $extraFactory,
        AccountFactory $accountFactory,
        \Magento\Framework\Filesystem $filesystem,
        SubscriberFactory $subscribersFactory,
        \Magento\Framework\Stdlib\DateTime\TimezoneInterface $timezone,
        \Magento\Checkout\Model\Session $checkoutSession
    ) {

        $this->emulation = $emulation;
        $this->checkoutSession = $checkoutSession;
        $this->configurableType = $configurableType;
        $this->imageHelper = $imageHelper;
        $this->productFactory = $productFactory;
        $this->categoryFactory = $categoryFactory;
        $this->egoiHelper = $egoiHelper;
        $this->orderFactory = $orderFactory;
        $this->scheduleFactory = $scheduleFactory;
        $this->coreRegistry = $coreRegistry;
        $this->accountFactory = $accountFactory;
        $this->scopeConfig = $scopeConfig;
        $this->storeManager = $storeManager;
        $this->listsFactory = $listsFactory;
        $this->extraFactory = $extraFactory;
        $this->customerFactory = $customerFactory;
        $this->subscriberFactory = $subscribersFactory;
        $this->authSession = $authSession;
        $this->coreSubscriberFactory = $coreSubscriberFactory;
        $this->coreCollectionFactory = $coreCollectionFactory;
        $this->filesystem = $filesystem;
        $this->url = $urlHelper;
        $this->reinitableConfig = $reinitableConfig;
        $this->config = $config;
        $this->timezone = $timezone;

        parent::__construct();
    }

    /**
     * @param $number
     * @param $message
     * @param $storeId
     *
     * @return bool|string
     */
    public function sendSMS($number, $message, $storeId)
    {

        if ( ! $number) {
            return false;
        }

        $from = $this->scopeConfig->getValue('egoi/info/sender', 'store', $storeId);

        $data = [
            "to"       => $number,
            "from"     => $from,
            "textBody" => $message,
            "encoding" => 'unicode',
            "maxCount" => 2,
        ];

        $result = $this->makeRequest(self::MOBILE_NUMBER_SEND_URL, [$data], 'POST');

        return ! empty($result[0]['messageId']) ? true : '. Server Response: '.$result['detail'];
    }

    /**
     * @param $document
     *
     * @return bool
     */
    public function getPhone($document)
    {

        if ( ! is_object($document) && stripos($document, '-') === false) {
            return false;
        }

        if ( ! is_object($document) && stripos($document, '-')) {
            #return $this->validateNumber($document);
        }

        if ($document instanceof Order) {
            $billing = $document->getBillingAddress();
        } elseif (is_object($document->getOrder())) {
            $billing = $document->getOrder()->getBillingAddress();
        }

        if ( ! isset($billing)) {
            return false;
        }

        $prefix = Subscriber::getPrefixForCountry($billing->getCountryId());

        $cellphoneField = str_replace(['addr_', 'ac_'], '', $this->scopeConfig->getValue('egoi/info/cellphone'));
        $number = preg_replace('/\D/', '', $billing->getData($cellphoneField));
        $number = ltrim($number, $prefix);
        $number = ltrim($number, 0);

        return $prefix.'-'.$number;
        //return $this->validateNumber($prefix.'-'.$number);
    }

    /**
     * @param $number
     *
     * @return bool
     */
    public function validateNumber($number)
    {

        if ( ! $number) {
            return false;
        }

        $number = preg_replace('/[^0-9\-]/', '', $number);

        $result = $this->makeRequest(self::MOBILE_NUMBER_VALIDATION_URL, ['number' => $number], 'POST');

        return ( ! isset($result['valid']) || $result['valid'] != 1) ? false : $number;
    }

    /**
     * @return bool
     */
    public function validateEgoiEnvironment()
    {

        $auth = $this->authSession->getUser()->getData('egoiAuth');

        if ($auth === true) {
            return true;
        }

        $info = $this->getUserData();

        if ( ! isset($info['general_info'])) {
            return false;
        }

        $account = $this->accountFactory->create()->getAccount();

        if ((int) $account->getData('cliente_id') == 0) {
            $details = $this->getAccountDetails();

            if ($account->getId() != $details['account_id']) {
                $account->getCollection()->walk('delete');
            }

            $account = $this->accountFactory->create()->getAccount();
            $account->setData($details)->save();

            if ((int) $account->getData('cliente_id') == 0) {
                return false;
            }
        }

        $this->authSession->getUser()->setData('egoiAuth', true);

        return true;
    }

    /**
     * @param $export
     *
     * @return array|mixed
     * @throws \Magento\Framework\Exception\LocalizedException
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function addSubscriberBulk($export = true)
    {

        $collection = $this->subscriberFactory->create()
            ->getCollection()
            ->addFieldToFilter('needs_sync', 1)
            ->setPageSize(500);

        if ($collection->getSize() < 2) {
            return [];
        }

        $totalRecords = $collection->getSize();

        $subscribers = [];
        $subscribersIds = [];
        /** @var Subscriber $subscriber */
        foreach ($collection as $subscriber) {
            $data = [];
            if ( ! filter_var($subscriber->getEmail(), FILTER_VALIDATE_EMAIL)) {
                continue;
            }
            if ( ! $subscriber->getData('uid')) {
                try {
                    $response = $this->createContactApi($subscriber->buildExtraFields($subscriber));
                    if (isset($response['contact_id'])) {
                        $this->setData('uid', $response['contact_id']);
                    } elseif ( ! empty($response['errors']['contacts'][0])) {
                        $this->setData('uid', $response['errors']['contacts'][0]);
                    }
                } catch (\Exception $e) {
                    $this->egoiHelper->getLogger()->critical($e->getMessage());
                }
            } else {
                $subscribers[] = $this->subscriberFactory->create()->buildExtraFields($subscriber);
            }
            $subscribersIds[] = $subscriber->getId();
        }

        if ($subscribersIds) {
            $this->egoiHelper->getConnection()
                ->update(
                    $subscriber->getResource()->getMainTable(),
                    [
                        'needs_sync' => 0,
                        'last_sync'  => $this->egoiHelper->getDateTime()->gmtDate(),
                    ]
                    , [
                        'subscriber_id IN (?)' => $subscribersIds,
                    ]
                );
        }

        if ($export) {
            return $subscribers;
        }

        $params = [
            'compare_field' => 'email',
            'mode'          => 'update',
            'contacts'      => $subscribers,
        ];

        return $this->makeRequest('lists/'.$this->getListId().'/contacts/actions/import-bulk', $params, 'POST');
    }

    /**
     * @param  null  $apiKey
     *
     * @return array|mixed
     */
    public function getAccountDetails($apiKey = null)
    {

        $details = $this->makeRequest('my-account', [], 'GET', $apiKey);

        if ( ! empty($details['general_info']['client_id'])) {
            return [
                'account_id'         => 1,
                'cliente_id'         => $details['general_info']['client_id'],
                'company_name'       => $details['general_info']['name'],
                'company_legal_name' => $details['billing_info']['company_legal_name'],
                'company_type'       => $details['billing_info']['type'],
                'country'            => $details['billing_info']['country']['country_code'],
                'state'              => $details['billing_info']['state'],
                'city'               => $details['billing_info']['city'],
                'address'            => $details['billing_info']['address1'],
                'website'            => $details['general_info']['website'],
                'credits'            => $details['balance_info']['balance'],
            ];
        }

        return $details;
    }

    /**
     * @return $this
     */
    public function getUserData()
    {

        return $this->makeRequest('my-account');
    }

    /**
     * @return mixed
     */
    public function getSMSSenders()
    {

        $senders = $this->makeRequest('senders/cellphone');
        if (isset($senders['items'])) {
            $senders = $senders['items'];
            foreach ($senders as $key => $sender) {
                if ( ! isset($sender['sender_id']) || $sender['status'] != 'active') {
                    unset($senders[$key]);
                }
            }

            return $senders;
        }

        return [];
    }

    /**
     * @param  null  $listId
     *
     * @return $this
     */
    public function getLists($listId = null)
    {

        $result = $this->makeRequest('lists');

        if (is_array($result['items'])) {
            $result = $result['items'];
            foreach ($result as $key => $value) {
                if ($listId && $listId != $value['list_id']) {
                    unset($result[$key]);
                    continue;
                }

                $result[$key]['extra_fields'] = $this->getExtraFields($value['list_id']);

                if (is_array($value) && (isset($value['extra_fields']) && ! is_array($value['extra_fields']))) {
                    continue;
                }
                if (isset($value['extra_fields']) && is_array($value['extra_fields'])) {
                    foreach ($value['extra_fields'] as $eKey => $eValue) {
                        unset($result[$key]['extra_fields'][$eKey]['list_id']);
                        unset($result[$key]['extra_fields'][$eKey]['options']);
                    }
                }
            }
        }

        return $result;
    }

    /**
     * @param $listId
     * @param $data
     *
     * @return mixed
     */
    public function updateList($listId, $data)
    {

        return $this->makeRequest('lists/'.$listId, $data, 'POST');
    }

    /**
     *
     */
    public function massSyncCustomers()
    {

        $this->addSubscriberBulk(false);
    }

    /**
     *
     */
    public function sync()
    {

        $account = $this->accountFactory->create()->getAccount();
        $key = $this->scopeConfig->getValue('egoi/info/api_key');

        if ( ! $key) {
            return;
        }
        $this->massSyncCustomers();
        $account->cron();
        $this->subscriberFactory->create()->cron();
    }

    /**
     * @param  \Magento\Store\Api\Data\StoreInterface  $store
     *
     * @return bool|string
     */
    public function createCatalog(\Magento\Store\Api\Data\StoreInterface $store)
    {

        $data = [];
        $data['title'] = 'Magento - '.$store->getName().' ['.$store->getId().']';
        $data['currency'] = $store->getBaseCurrencyCode();
        $data['language'] = $this->scopeConfig->getValue('general/locale/code', 'store', $store->getId());

        $data['language'] = explode('_', $data['language']);

        if (count($data['language']) == 2) {
            $data['language'] = $data['language'][0];
        } elseif (count($data['language']) == 3) {
            $data['language'] = $data['language'][2];
        }
        $data['language'] = strtolower($data['language']);

        return $this->makeRequest('catalogs', $data, 'POST');
    }

    /**
     *
     */
    public function checkEcommerceCatalogs()
    {

        $stores = $this->getEcommerceCatalogs();

        foreach ($this->storeManager->getStores() as $store) {
            $catalogIdForStore = $this->scopeConfig->getValue('egoi/products/catalog_id', 'stores', $store->getId());
            if ( ! $catalogIdForStore || ! array_key_exists($catalogIdForStore, $stores)) {
                $result = $this->createCatalog($store);
                $this->config->saveConfig(
                    'egoi/products/catalog_id',
                    $result['catalog_id'],
                    'stores',
                    $store->getId()
                );
                $this->config->saveConfig('egoi/products/enable', 1, 'stores', $store->getId());
                $this->reinitableConfig->reinit();
            }
        }
    }

    /**
     *
     */
    public function getEcommerceCatalogs()
    {

        $response = $this->makeRequest('catalogs');

        $catalogs = [];

        if ($response['total_items'] > 0) {
            foreach ($response['items'] as $info) {
                $catalogs[$info['catalog_id']] = $info['title'].' - '.$info['language'].' - '.$info['currency'];
            }
        }

        return $catalogs;
    }

    /**
     *
     */
    public function getActiveEcommerceCatalogs()
    {

        $stores = $this->getEcommerceCatalogs();
        $return = [];
        foreach ($this->storeManager->getStores() as $store) {
            $catalogIdForStore = $this->scopeConfig->getValue('egoi/products/catalog_id', 'store', $store->getId());
            if ($catalogIdForStore && array_key_exists($catalogIdForStore, $stores)) {
                $return[$store->getId()] = $stores[$catalogIdForStore];
            }
        }

        return $return;
    }

    /**
     * @param  int  $storeId
     *
     * @return bool
     */
    public function buildProductsFeed($storeId = 1, $productsIds = [])
    {

        $this->storeManager->getStore()->setId($storeId);
        $attributesInfo = $this->scopeConfig->getValue('egoi/products', 'stores', $storeId);
        $catalogId = $attributesInfo['catalog_id'];

        if (empty($attributesInfo['enable'])) {
            return false;
        }

        $qtyToSync = 25000;
        $category = $this->categoryFactory->create();
        $tree = $category->getTreeModel();
        $tree->load();

        $ids = $tree->getCollection()->addIsActiveFilter()->getAllIds();
        $cnt = 0;
        $categories = [];
        $catInfo = [];
        if ($ids) {
            foreach ($ids as $k => $id) {
                $category->load($id);
                $categories[$id]['name'] = $category->getName();
                $categories[$id]['path'] = $category->getPath();
            }

            foreach ($ids as $id) {
                $path = explode('/', $categories[$id]['path']);
                $string = '';
                foreach ($path as $pathId) {
                    if (isset($categories[$pathId])) {
                        $string .= $categories[$pathId]['name'].' / ';
                        $cnt++;
                    }
                }

                $string = str_replace('Root Catalog / Default Category / ', '', $string);
                $string = str_replace('Default Category / ', '', $string);
                $catInfo[$id] = rtrim($string, ' / ');
            }
        }

        $currentPage = 1;
        while (true) {
            /** @var \Magento\Catalog\Model\ResourceModel\Product\Collection $collection */
            $collection = $this->productFactory->create()
                ->getCollection()
                ->addAttributeToSelect('*');

            if ($productsIds) {
                $collection->addFieldToFilter('entity_id', ['in' => $productsIds]);
            }

            $collection->addAttributeToSelect('price')
                ->addAttributeToFilter('status', 1)
                ->addAttributeToFilter('visibility', 4)
                ->setOrder('entity_id', 'ASC')
                ->setPageSize($qtyToSync)
                ->setCurPage($currentPage);

            $return = [];

            /** @var \Magento\Catalog\Model\Product $product */
            foreach ($collection as $product) {
                //define area according to your requirement like frontend or adminhtml
                $this->emulation->startEnvironmentEmulation($storeId, \Magento\Framework\App\Area::AREA_FRONTEND, true);
                $product->setStoreId($storeId);
                $parents = $this->configurableType->getParentIdsByChild($product->getId());

                if ($parents && ! $this->scopeConfig->isSetFlag('egoi/products/configurable')) {
                    continue;
                }

                $productUrl = $product->getProductUrl();
                if ($parents) {
                    $parent = $this->productFactory->create();
                    $productImg = $parent->load($parents[0])->setStoreId($storeId);
                    $productUrl = $parent->getProductUrl();
                    $imageUrl = $this->imageHelper->init($productImg, 'product_page_image_small')
                        ->setImageFile($productImg->getSmallImage())
                        ->getUrl();
                } else {
                    $imageUrl = $this->imageHelper->init($product, 'product_page_image_small')
                        ->setImageFile($product->getSmallImage())
                        ->getUrl();
                }

                $this->emulation->stopEnvironmentEmulation();
                $return[$product->getId()]['product_identifier'] = $product->getId();
                $return[$product->getId()]['sku'] = $product->getSku();
                $return[$product->getId()]['name'] = $product->getName();
                $return[$product->getId()]['description'] = $product->getShortDescription();
                $return[$product->getId()]['price'] = number_format($product->getPrice(), 2, '.', '');
                $return[$product->getId()]['image_link'] = $imageUrl;
                $return[$product->getId()]['link'] = $productUrl;

                if ($product->getFinalPrice() < $product->getPrice()) {
                    $return[$product->getId()]['sale_price'] = number_format($product->getFinalPrice(), 2, '.', '');
                }

                $cats = $product->getCategoryIds();

                foreach ($cats as $cat) {
                    if (isset($catInfo[$cat])) {
                        $return[$product->getId()]['categories'][] = $catInfo[$cat];
                    }
                }

                $related = $product->getRelatedProductCollection();

                foreach ($related as $info) {
                    $return[$product->getId()]['related_product'][] = $info->getSku();
                }

                $attributes = $product->getAttributes();

                foreach ($attributes as $attribute) {
                    if ( ! in_array($attribute->getAttributeCode(), $attributesInfo)) {
                        continue;
                    }

                    if ($attribute->getSource()) {
                        $value = $product->getResource()
                            ->getAttribute($attribute->getAttributeCode())
                            ->getFrontend()
                            ->getValue($product);

                        $return[$product->getId()][array_search(
                            $attribute->getAttributeCode(),
                            $attributesInfo
                        )]
                            = (string) $value;
                    }
                }
            }

            $return = ['products' => $return];

            if ($productsIds) {
                return $return;
            }
            $response = $this->makeRequest('catalogs/'.$catalogId.'/products/actions/import', $return, 'POST');

            if (empty($response['success'])) {
                return false;
            }

            if ($collection->count() < $qtyToSync) {
                return true;
            }
            $currentPage++;
        }

        return true;
    }

    /**
     * @param $domain
     *
     * @return mixed
     */
    public function assignConnectedSite($domain)
    {

        $response = $this->makeRequest("connectedsites/".$domain);

        if (isset($response['code'])) {
            $this->config->saveConfig('egoi/info/connected_code', $response['code'], 'default', '0');
            $this->reinitableConfig->reinit();
        }

        return $response;
    }

    /**
     * @return array|mixed
     */
    public function getConnectedSiteList()
    {

        $response = $this->makeRequest('connectedsites');

        if (isset($response['items'])) {
            return $response['items'];
        }

        return [];
    }

    /**
     * @return void
     */
    public function createConnectedSite()
    {

        $site = rtrim($this->scopeConfig->getValue('web/unsecure/base_url'), '/');
        $listId = $this->listsFactory->create()->getList()->getListnum();

        $response = $this->makeRequest('connectedsites', ['domain' => $site, 'list_id' => $listId], 'POST');

        if (isset($response['domain'])) {
            $this->config->saveConfig('egoi/info/connected', $response['domain'], 'default', '0');
            $this->reinitableConfig->reinit();
            $this->assignConnectedSite($response['domain']);
        }
    }

    /**
     * @param $uid
     *
     * @return mixed
     */
    public function unsubscribeApi($uid)
    {

        $data = ['data' => ['contact_id' => $uid, 'unsubscription_method' => 'manual']];
        $listId = $this->getListId();

        $result = $this->makeRequest(
            "lists/".$listId."/contacts/actions/unsubscribe",
            $data,
            'POST'
        );
        $this->checkoutSession->setData('egoi_subscriber', null);
        if ( ! isset($result['success'])) {
            return false;
        }

        return true;
    }

    /**
     * @param $email
     *
     * @return array
     */
    public function getSubscriberStatusApi($email)
    {

        $return = $this->coreRegistry->registry('egoi_subscriber');

        if ( ! empty($return['status'])) {
            return $return;
        }

        $attempts = (int) $this->checkoutSession->getData('egoi_subscriber_attemps');

        if ($attempts < 2) {
            $return = [
                'status' => false,
                'email'  => false,
                'uid'    => false,
            ];
            $listId = $this->getListId();

            try {
                $response = $this->makeRequest("lists/$listId/contacts?email=".urlencode($email));

                if ( ! empty($response['items'][0]['base']['status'])) {
                    $return = [
                        'status' => $response['items'][0]['base']['status'],
                        'email'  => $response['items'][0]['base']['email'],
                        'uid'    => $response['items'][0]['base']['contact_id'],
                    ];
                }
            } catch (\Exception $e) {
                $this->egoiHelper->getLogger()->critical($e->getMessage());
            }
            $this->checkoutSession->setData('egoi_subscriber_attemps', $attempts + 1);
            $this->checkoutSession->setData('egoi_subscriber', $return);
        }

        if (is_array($return)) {
            $this->coreRegistry->register('egoi_subscriber', $return, true);
        }

        return $this->checkoutSession->getData('egoi_subscriber');
    }

    /**
     * @param $email
     *
     * @return mixed
     */
    public function getSubscriberApi($email)
    {

        $listId = $this->getListId();
        try {
            $response = $this->makeRequest("lists/$listId/contacts?email=".urlencode($email));
            if ( ! empty($response['items'][0]['base']['status'])) {
                return $response['items'][0]['base'];
            }
        } catch (\Exception $e) {
            return false;
        }

        return false;
    }

    /**
     * @param $uid
     * @param $status
     *
     * @return mixed
     */
    public function updateContactStatusApi($uid, $status)
    {

        $data = [];
        $data['base']['status'] = $status;

        return $this->makeRequest(
            "lists/{$this->getListId()}/contacts/".$uid,
            $data,
            'PATCH'
        );
    }

    /**
     * @param $uid
     * @param $status
     *
     * @return mixed
     */
    public function updateContact($uid, $data)
    {

        return $this->makeRequest(
            "lists/{$this->getListId()}/contacts/".$uid,
            $data,
            'PATCH'
        );
    }

    /**
     * @param $uid
     * @param $status
     *
     * @return mixed
     */
    public function removeContact($uid)
    {

        $this->checkoutSession->setData('egoi_subscriber', null);
        $data['base'] = ['status' => 'removed'];

        return $this->makeRequest("lists/{$this->getListId()}/contacts/".$uid, $data, 'PATCH');
    }

    /**
     * @param $data
     *
     * @return false|mixed
     */
    public function createContactApi($data)
    {

        $response = $this->makeRequest("lists/{$this->getListId()}/contacts", $data, 'POST');

        if ( ! empty($response['contact_id'])) {
            return $response['contact_id'];
        }

        return false;
    }

    /**
     * @param $listId
     *
     * @return mixed
     */
    public function getExtraFields($listId)
    {

        $extraFields = $this->makeRequest("lists/$listId/fields");

        foreach ($extraFields as $key => $extraField) {
            if ($extraField['type'] != 'extra') {
                unset($extraFields[$key]);
            }
        }

        return $extraFields;
    }

    /**
     * @param $endpoint
     * @param $data
     * @param $method
     * @param $key
     *
     * @return mixed
     */
    public function makeRequest($endpoint, $data = [], $method = 'GET', $key = null)
    {

        if ($this->scopeConfig->isSetFlag('egoi/test/log')) {
            $this->egoiHelper->getLogger()->info(
                'EGOI START: '.@$_SERVER['REQUEST_URI'].$endpoint.'|'.$method.'|'.print_r($data, true)
            );
        }

        $curl = curl_init();

        $apiKey = $this->scopeConfig->getValue('egoi/info/api_key');
        if ($key) {
            $apiKey = $key;
        }

        $suffix = '';
        if (stripos($endpoint, 'http') === false) {
            $suffix = self::ECOMMERCE_URL;
        }

        $dataEndpoint = [
            CURLOPT_URL            => $suffix.$endpoint,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING       => "",
            CURLOPT_MAXREDIRS      => 5,
            CURLOPT_TIMEOUT        => 500,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTP_VERSION   => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST  => strtoupper($method),
            CURLOPT_HTTPHEADER     => [
                "Apikey: ".$apiKey,
                "Content-Type: application/json",
                "PluginKey: ".self::PLUGIN_KEY,
            ],
        ];

        if ($data) {
            $dataEndpoint[CURLOPT_POSTFIELDS] = json_encode($data);
        }

        curl_setopt_array($curl, $dataEndpoint);

        $response = json_decode(curl_exec($curl), true);
        curl_close($curl);

        if ($this->scopeConfig->isSetFlag('egoi/test/log')) {
            $this->egoiHelper->getLogger()->info(
                'EGOI END: '.@$_SERVER['REQUEST_URI'].$endpoint.'|'.$method.'|'.print_r(
                    $data,
                    true
                ).'|'.print_r($response, true)
            );
        }

        return $response;
    }

    /**
     * @return string
     */
    public function getListId()
    {

        $resource = $this->productFactory->create()->getResource();
        $name = $resource->getTable('egoi_lists');

        return $resource->getConnection()->fetchOne("SELECT listnum FROM ".$name." LIMIT 1");
    }

    /**
     * @param     $listId
     * @param  int  $start
     * @param  int  $limit
     *
     * @return mixed
     */
    public function getSubscribers($listId, $start = 0, $limit = 100)
    {

        return $this->makeRequest("lists/$listId/contacts?offset=$start&limit=$limit");
    }

    /**
     * @param $listId
     * @param $uid
     *
     * @return mixed
     */
    public function getSubscriber($listId, $uid)
    {

        return $this->makeRequest("lists/$listId/contacts/".$uid);
    }

    /**
     * @return mixed
     */
    public function getWebHook()
    {

        $listId = $this->getListId();
        $existing = $this->makeRequest("webhooks");
        $url = $this->scopeConfig->getValue('web/secure/base_url').'egoi/callback';
        if ( ! empty($existing['items'])) {
            foreach ($existing['items'] as $callback) {
                if ($callback['url'] == $url && $callback['list_id'] == $listId) {
                    return $callback;
                }
            }
        }

        $data = [];
        $data['list_id'] = $listId;
        $data['url'] = $url;
        $data['fields'] = ['email'];
        $data['actions'] = [
            "unsubscribe",
            "subscription",
            "edit_subscription",
            "resubscription",
        ];

        return $this->makeRequest("webhooks", $data, 'POST');
    }

    /**
     * @param $orderId
     *
     * @return bool|void
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function syncOrder($orderId)
    {

        if ( ! $this->scopeConfig->isSetFlag('egoi/info/sync_orders')) {
            return false;
        }

        $listId = $this->getListId();
        if ( ! $listId) {
            return false;
        }

        $order = $this->orderFactory->create()->load($orderId);
        if ( ! $order->getId()) {
            return false;
        }
        $sub = $this->subscriberFactory->create()->loadByEmail($order->getCustomerEmail());
        if ( ! $sub) {
            return false;
        }

        $product = $this->productFactory->create()->load(1);

        $json = [];
        $url = parse_url($this->storeManager->getStore($order->getStoreId())->getBaseUrl())['host'];
        $data = [
            'order_id'     => $order->getIncrementId(),
            'order_status' => $this->orderStateMapping[$order->getState()] ?? 'unknown',
            'order_total'  => round($order->getBaseGrandTotal()),
            'date'         => $order->getCreatedAt(),
            'contact'      => ['base' => ['contact_id' => $sub->getData('uid')]],
        ];

        $productIds = [];
        /** @var \Magento\Sales\Model\Order\Item $item */
        foreach ($order->getAllVisibleItems() as $item) {
            $productIds[] = $item->getProductId();
            $data['products'][$item->getItemId().'-'.$item->getProductId()] = [
                'product_identifier' => $item->getProductId(),
                'name'               => $item->getName(),
                'sku'                => $item->getSku(),
                'description'        => $item->getProduct()->getShortDescription(),
                'sale_price'         => $item->getBasePrice(),
                'quantity'           => round($item->getQtyOrdered()),
            ];
        }

        $cat = $this->buildProductsFeed($order->getStoreId(), $productIds);
        foreach ($data['products'] as $key => $item) {
            if ( ! is_string($key)) {
                continue;
            }
            $pid = explode('-', $key);
            $pid = $pid[1];

            if ( ! empty($cat['products'][$pid])) {
                $data['products'][$key]['price'] = $cat['products'][$pid]['price'];
                $data['products'][$key]['image_link'] = $cat['products'][$pid]['image_link'];
                $data['products'][$key]['categories'] = array_filter($cat['products'][$pid]['categories']);
            }
        }

        if ($data) {
            $result = $this->makeRequest($url."/orders", $data, 'POST');
            if ( ! empty($result['result']) && $result['result'] == 'success') {
                $order->getResource()
                    ->getConnection()
                    ->update(
                        $order->getResource()->getTable('sales_order'),
                        [
                            'egoi_synced' => 1,
                        ],
                        [
                            'entity_id=?' => $orderId,
                        ]);
            }

            return true;
        }
    }

    public function syncOrders()
    {

        if ( ! $this->scopeConfig->isSetFlag('egoi/info/sync_orders')) {
            return false;
        }
        $installDate = false;
        if ( ! $this->scopeConfig->isSetFlag('egoi/info/sync_orders_history')) {
            $installDate = $this->scopeConfig->getValue('egoi/install/date');
            if ($installDate) {
                $d = \DateTime::createFromFormat('Y-m-d H:i:s', $installDate);
                if ( ! $d || $d->format('Y-m-d H:i:s') != $installDate) {
                    $installDate = null;
                }
            } else {
                $installDate = '2025-05-01 00:00:00';
            }
        }

        $listId = $this->getListId();
        if ( ! $listId) {
            return false;
        }


        $json = [];
        $orderIds = [];
        $orders = $this->orderFactory->create()
            ->getCollection()
            ->addFieldToFilter('egoi_synced', 0)
            ->addFieldToFilter('base_grand_total', ['gt' => 0])
            ->setPageSize(250);
        if ($installDate) {
            $orders->addFieldToFilter('created_at', ['gteq' => $installDate]);
        }

        /** @var Order $order */
        $tmpHold = [];
        foreach ($orders as $order) {
            $email = $order->getCustomerEmail();
            $uid = false;
            if ( ! array_key_exists($email, $tmpHold)) {
                $sub = $this->subscriberFactory->create()->loadByEmail($email);
                $uid = $sub->getData('uid');
                if ( ! $uid) {
                    $exists = $this->getSubscriberApi($sub->getEmail());
                    if ($exists) {
                        $sub->setData('uid', $exists['contact_id'])->save();
                        $uid = $exists['contact_id'];
                    }
                }
                $tmpHold[$email] = $uid;
            } else {
                $uid = $tmpHold[$email];
            }
            if ($uid) {
                $url = parse_url($this->storeManager->getStore($order->getStoreId())->getBaseUrl())['host'];
                $data = [
                    'contact_id'   => $uid,
                    'store_url'    => $url,
                    'order_id'     => $order->getIncrementId(),
                    'order_status' => $this->orderStateMapping[$order->getState()] ?? 'unknown',
                    'revenue'      => round($order->getBaseGrandTotal()),
                    'date'         => $order->getCreatedAt(),
                ];

                /** @var \Magento\Sales\Model\Order\Item $item */
                foreach ($order->getAllVisibleItems() as $item) {
                    $productCat = '';

                    if ($item->getProduct()->getCategoryCollection()) {
                        $cat = $item->getProduct()->getCategoryCollection()->getLastItem();
                        if ($cat->getId() > 2) {
                        $productCat = '['.$cat->getId().']'.$cat->getName();
                    }
                    }

                    $data['items'][] = [
                        'id'       => $item->getSku(),
                        'name'     => $item->getName(),
                        'category' => $productCat,
                        'price'    => $item->getBasePrice(),
                        'qty'      => round($item->getQtyOrdered()),
                    ];
                }

                $json[] = $data;
            }

            $orderIds[] = $order->getId();
        }

        if ($json) {
            $result = $this->makeRequest("lists/$listId/orders", $json, 'POST');

            if ( ! empty($result['result']) && $result['result'] == 'success') {
                foreach ($orderIds as $orderId) {
                    $order->getResource()
                        ->getConnection()
                        ->update(
                            $order->getResource()->getTable('sales_order'),
                            [
                                'egoi_synced' => 1,
                            ],
                            [
                                'entity_id=?' => $orderId,
                            ]);
                }

                return true;
            } else {
                $this->egoiHelper->getLogger()->critical(json_encode($result));
            }
        }

        return false;
    }

    public function getHelper()
    {

        return $this->egoiHelper;
    }

}
