<?php

namespace Egoi\Marketing\Observer;

use Magento\Framework\Event\ObserverInterface;

/**
 * Class ChangeStatus
 *
 * @package Egoi\Marketing\Observer
 */
class ChangeStatus implements ObserverInterface
{

    /**
     * @var \Egoi\Marketing\Model\SubscriberFactory
     */
    protected $subscriberFactory;

    /**
     * @var \Egoi\Marketing\Model\AutorespondersFactory
     */
    protected $_autorespondersFactory;

    /**
     * @param  \Egoi\Marketing\Model\SubscriberFactory  $subscriberFactory
     * @param  \Egoi\Marketing\Model\AutorespondersFactory  $autorespondersFactory
     */
    function __construct(
        \Egoi\Marketing\Model\SubscriberFactory $subscriberFactory,
        \Egoi\Marketing\Model\AutorespondersFactory $autorespondersFactory
    ) {

        $this->_autorespondersFactory = $autorespondersFactory;
        $this->subscriberFactory = $subscriberFactory;
    }

    /**
     * @param  \Magento\Framework\Event\Observer  $event
     */
    public function execute(\Magento\Framework\Event\Observer $event)
    {

        try {
            /** @var \Magento\Sales\Model\Order $order */
            $order = $event->getEvent()->getOrder();
            if ($order->getState() != \Magento\Sales\Model\Order::STATE_NEW
                && $order->getData('state')
                && $order->getOrigData('state') != $order->getData('state')
            ) {
                $subscriber = $this->subscriberFactory->create()->loadByEmail($order->getCustomerEmail());
                if ($subscriber->getId()) {
                    $order->setData('egoi_synced', 0);
                }
            }

            $this->_autorespondersFactory->create()->changeStatus($event);
        } catch (\Exception $e) {
        }
    }

}
