<?php

namespace Egoi\Marketing\Observer;

use Magento\Framework\Event\ObserverInterface;

/**
 * Class NewOrder
 *
 * @package Egoi\Marketing\Observer
 */
class NewOrder implements ObserverInterface
{

    /**
     * @var \Egoi\Marketing\Model\ConversionsFactory
     */
    protected $conversionsFactory;

    /**
     * @var \Egoi\Marketing\Model\AutorespondersFactory
     */
    protected $autorespondersFactory;

    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * @var \Egoi\Marketing\Model\SubscriberFactory
     */
    protected $subscriberFactory;

    /**
     * @param  \Egoi\Marketing\Model\SubscriberFactory  $subscriberFactory
     * @param  \Egoi\Marketing\Model\ConversionsFactory  $conversionsFactory
     * @param  \Magento\Framework\App\Config\ScopeConfigInterface  $scopeConfig
     * @param  \Egoi\Marketing\Model\AutorespondersFactory  $autorespondersFactory
     */
    function __construct(
        \Egoi\Marketing\Model\SubscriberFactory $subscriberFactory,
        \Egoi\Marketing\Model\ConversionsFactory $conversionsFactory,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Egoi\Marketing\Model\AutorespondersFactory $autorespondersFactory
    ) {

        $this->subscriberFactory = $subscriberFactory;
        $this->conversionsFactory = $conversionsFactory;
        $this->scopeConfig = $scopeConfig;
        $this->autorespondersFactory = $autorespondersFactory;
    }

    /**
     * @param  \Magento\Framework\Event\Observer  $event
     */
    public function execute(\Magento\Framework\Event\Observer $event)
    {

        try {
            $this->autorespondersFactory->create()->newOrder($event);
            $this->conversionsFactory->create()->newConversion($event);

            /** @var \Magento\Sales\Model\Order $order */
            $order = $event->getEvent()->getOrder();

            if ($this->scopeConfig->getValue('egoi/info/auto')) {
                $this->subscriberFactory->create()
                    ->subscribe($order->getCustomerEmail(), false, $order->getStoreId());
            }
        } catch (\Exception $e) {
        }
    }

}
